//
//  Solar Guide Lamp  0.7  /  main.c
//      Part reduction edition
//  Copyright(C)2011 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		Solar cell to battery charge and night marker lamp.
//		Charging method is direct.
//		SMP(Switch mode pump: Boost DC-DC converter) include.
//
//	Usage
//		Connect solar cell(ETM250-2V).
//		Insert AAA x 1 rechargeble battery.
//
//  Device: PIC16F1823
//
//  Pin assign:
//		p1:  Vdd                                             +3.6V 
//		p2:  RA5/T1CKI/T1OSI/OSC1/CLKIN                      -
//		p3:  RA4/T1G/SDO/CLKR/T1OSO/CLKOUT/OSC2/CPS3/AN3     On/Shutdown
//		p4:  RA3/MCLR/VPP/T1G/SS                             ISP 
//		p5:  RC5/MDCIN2/RX/DT/CCP1/P1A                       PFM(SMP)
//		p6:  RC4/MDOUT/TX/CK/P1B/SRNQ/C2OUT                  -
//		p7:  RC3/MDMIN/SS/P1C/C12IN3-/CPS7/AN7               LED-Green
//		p8:  RC2/AN6/CPS6/C12IN2-/P1D/SDO/MDCIN1             LED-Red
//		p9:  RC1/AN5/CPS5/C12IN1-/SDA/SDI                    RA0
//		p10: RC0/AN4/CPS4/C2IN+/SCL/SCK                      Vbatt/Vsol
//		p11: RA2/AN2/CPS2/T0CKI/INT/C1OUT/SRQ/FLT0           LED-Blue
//		p12: RA1/AN1/CPS1/C12IN0-/VREF/SRI/RX/DT/ICSPCLK     ISP
//		p13: RA0/AN0/CPS0/C1IN+/DACOUT/TX/CK/ICSPDAT/ICDDAT  RC1,ISP
//		p14: Vss                                             GND
//
//	Clock:	Fosc=8MHz
//
//	Internal peripheral modules:
//		BOR:	-
//		Timer0:	-
//		Timer1:	-
//		Timer2:	PWM mode
//		ECCP1:	PWM mode
//		MSSP1:	-
//		EUSART:	-
//		ADC:	Measuering voltage
//		DAC:	Connect to Comp1
//		FVR:	Connect to Comp1
//		Comp1:	Connect to ECCP1(PWM)
//		Comp2:	-
//		SRLatch:-
//		DSM:    -
//		CPS:    -
//
//	Environment:
//		IDE: MPLAB IDE 8.63
//      HI-TECH C Compiler for PIC10/12/16 MCUs (Lite Mode)  V9.81
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2011.10.02 v0.7  First release
//

#include "htc.h"						// HI-TECH C


//*********************************************************************
//
//	Definition
//
//*********************************************************************

//
//	CPU Configuration Fuses bits
//
__CONFIG(
//	FOSC_ECH	&	// ECH, External Clock, High Power Mode (4-32 MHz): device clock supplied to CLKIN pin
//	FOSC_ECM	&	// ECM, External Clock, Medium Power Mode (0.5-4 MHz): device clock supplied to CLKIN pin
//	FOSC_ECL	&	// ECL, External Clock, Low Power Mode (0-0.5 MHz): device clock supplied to CLKIN pin
	FOSC_INTOSC	&	// INTOSC oscillator: I/O function on CLKIN pin
//	FOSC_EXTRC	&	// EXTRC oscillator: External RC circuit connected to CLKIN pin
//	FOSC_HS		&	// HS Oscillator, High-speed crystal/resonator connected between OSC1 and OSC2 pins
//	FOSC_XT		&	// XT Oscillator, Crystal/resonator connected between OSC1 and OSC2 pins
//	FOSC_LP		&	// LP Oscillator, Low-power crystal connected between OSC1 and OSC2 pins
//	WDTE_ON		&	// Watchdog Timer Enable
//	WDTE_NSLEEP	&	// WDT enabled while running and disabled in Sleep
//	WDTE_SWDTEN	&	// WDT controlled by the SWDTEN bit in the WDTCON register
	WDTE_OFF	&	// WDT disabled
	PWRTE_OFF	&	// PWRT disabled
//	PWRTE_ON	&	// PWRT enabled
//	MCLRE_ON	&	// MCLR/VPP pin function is MCLR
	MCLRE_OFF	&	// MCLR/VPP pin function is digital input
	CP_OFF		&	// Program memory code protection is disabled
//	CP_ON		&	// Program memory code protection is enabled
	CPD_OFF		&	// Data memory code protection is disabled
//	CPD_ON		&	// Data memory code protection is enabled
//	BOREN_ON	&	// Brown-out Reset enabled
//	BOREN_NSLEEP&	// Brown-out Reset enabled while running and disabled in Sleep
//	BOREN_SBODEN&	// Brown-out Reset controlled by the SBOREN bit in the BORCON register
	BOREN_OFF	&	// Brown-out Reset disabled
	CLKOUTEN_OFF&	// CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin
//	CLKOUTEN_ON	&	// CLKOUT function is enabled on the CLKOUT pin
//	IESO_ON		&	// Internal/External Switchover mode is enabled
	IESO_OFF	&	// Internal/External Switchover mode is disabled
//	FCMEN_ON	&	// Fail-Safe Clock Monitor is enabled
	FCMEN_OFF	&	// Fail-Safe Clock Monitor is disabled
	0x3fff
);
__CONFIG(
	WRT_OFF		&	// Write protection off
//	WRT_BOOT	&	// 000h to 1FFh write protected, 200h to 7FFh may be modified by EECON control
//	WRT_HALF	&	// 000h to 3FFh write protected, 400h to 7FFh may be modified by EECON control
//	WRT_ALL		&	// 000h to 7FFh write protected, no addresses may be modified by EECON control
//	PLLEN_ON	&	// 4x PLL enabled
	PLLEN_OFF	&	// 4x PLL disabled
	STVREN_ON	&	// Stack Overflow or Underflow will cause a Reset
//	STVREN_OFF	&	// Stack Overflow or Underflow will not cause a Reset
	BORV_19		&	// Brown-out Reset Voltage (VBOR) set to 1.9 V
//	BORV_25		&	// Brown-out Reset Voltage (VBOR) set to 2.5 V
//	LVP_ON		&	// Low-Voltage Programming Enable
	LVP_OFF		&	// High-voltage on MCLR/VPP must be used for programming
	0x3fff
);

//
// Application constant
//
//#define _XTAL_FREQ		8000000		// [Hz]	-> IDE project parameter
#define ADC_VREF			3.6			// [V]
#define ADC_RESOLUTION		1024		// 10bit
#define V2AD(v)				((short)(v/ADC_VREF*ADC_RESOLUTION))
#define BATT_LOWLIM			V2AD(1.15)	// [V]
#define BATT_HIGHLIM		V2AD(1.45)	// [V]
//#define LED_GENERATOR_LOAD				// 

//
//	LED signal   H_________________H_________________H_________________HH________________HH________________HH________________ ... HHHHHHHHHHHHHHHHH_HHHHHHHHHHHHHHHHH_HHHHHHHHHHHHHHHHH_
//
//	             <---PWM_PERIOD--->                                     
//	             <----------------LIGHT_PWM_TIME----------------------> 
//	             <--------------------------------------------------------------------------LIGHT_GRADUATION--------------------------------------------------------------------------->
//
#define LIGHT_PWM_PERIOD	1024		// Program loop
#define LIGHT_PWM_TIME		3			// Pulse count
#define LIGHT_GRADUATION	32			// Bright level "dutytbl[]"
//
//	   *       *       *                *       *       *    
//	  ***     ***     ***              ***     ***     ***   
//	 *****   *****   *****            *****   *****   *****  
//	******* ******* *******          ******* ******* *******
//
//  <------LIGHT_WAVE-----><--1sec--><------LIGHT_WAVE-----><--1sec--> ...
//	<----------------------LIGHT_ONTIME----------------------------------->
//
#define LIGHT_WAVE			2			// 1 unit wave
#define LIGHT_ONTIME		1200		// Unit count

//#define NIGHT_TH			0.09		// Ratio of the sunrise brightness
//#define NIGHT_TH			0.10		// Ratio of the sunrise brightness
#define NIGHT_TH			0.11		// Ratio of the sunrise brightness
//#define NIGHT_TH			0.12		// Ratio of the sunrise brightness
//#define NIGHT_TH			0.13		// Ratio of the sunrise brightness

#define CHARGE_LIM			(60*60*24)	// [sec] Fail safe check

// Digital output port assign
#define LED_RED_ON()		LATCbits.LATC2=0
#define LED_RED_OFF()		LATCbits.LATC2=1
#define LED_RED_REV()		LATCbits.LATC2^=1
#define LED_RED_INIT()		LED_RED_OFF(), TRISCbits.TRISC2=0
#define LED_RED_TERM()		TRISCbits.TRISC2=1

#define LED_GREEN_ON()		LATCbits.LATC3=0
#define LED_GREEN_OFF()		LATCbits.LATC3=1
#define LED_GREEN_REV()		LATCbits.LATC3^=1
#define LED_GREEN_INIT()	LED_GREEN_OFF(), TRISCbits.TRISC3=0
#define LED_GREEN_TERM()	TRISCbits.TRISC3=1

#define LED_BLUE_ON()		LATAbits.LATA2=0
#define LED_BLUE_OFF()		LATAbits.LATA2=1
#define LED_BLUE_REV()		LATAbits.LATA2^=1
#define LED_BLUE_INIT()		LED_BLUE_OFF(), TRISAbits.TRISA2=0
#define LED_BLUE_TERM()		TRISAbits.TRISA2=1

#define SELF_POWER_ON()		LATAbits.LATA4=1
#define SELF_POWER_OFF()	LATAbits.LATA4=0
#define SELF_POWER_REV()	LATAbits.LATA4^=1
#define SELF_POWER_INIT()	SELF_POWER_ON(), TRISAbits.TRISA4=0
#define SELF_POWER_TERM()	TRISAbits.TRISA4=1


//
// Global area
//
const int dutytbl[] = {					// PWM duty table (off time)
   1024, 512, 341, 256, 204, 170, 146, 124,
    108,  90,  76,  64,  55,  47,  40,  34,
     29,  25,  21,  18,  15,  13,  11,   9,
      8,   7,   6,   5,   4,   3,   2,   1};
#define COLOR_VBAT_NUM 12				// Table entry
const           int vbat2code[]  = {V2AD(1.36),V2AD(1.33),V2AD(1.32),V2AD(1.31),V2AD(1.30),V2AD(1.29),V2AD(1.28),V2AD(1.27),V2AD(1.26),V2AD(1.25),V2AD(1.24)      };
const unsigned char vbat2red[]   = {        31,        20,         0,         0,         0,        20,        31,        31,        31,        31,        31,   0 };
const unsigned char vbat2green[] = {        31,        31,        31,        31,        31,        31,        31,        20,         0,         0,         0,   0 };
const unsigned char vbat2blue[]  = {        31,        31,        31,        15,         0,         0,         0,         0,         0,        15,        31,  31 };
//                                  White      LightSky   LightBlue  BlueGreen  Green      Lime       Yellow     Orange     Red        Pink       Violet     Blue


//*********************************************************************
//
//	Utilitys
//
//*********************************************************************

//---------------------------------------------------------------------
//	A/D converter
//---------------------------------------------------------------------
void adc_start() {
	// Setup chip pin
//	ANSELAbits.ANSA0 = 1;				/* AN0 */
//	TRISAbits.TRISA0 = 1;				/* AN0 */
//	ANSELAbits.ANSA1 = 1;				/* AN1 */
//	TRISAbits.TRISA1 = 1;				/* AN1 */
//	ANSELAbits.ANSA2 = 1;				/* AN2 */
//	TRISAbits.TRISA2 = 1;				/* AN2 */
//	ANSELAbits.ANSA4 = 1;				/* AN3 */
//	TRISAbits.TRISA4 = 1;				/* AN3 */
	ANSELCbits.ANSC0 = 1;				/* AN4 */
	TRISCbits.TRISC0 = 1;				/* AN4 */
//	ANSELCbits.ANSC1 = 1;				/* AN5 */
//	TRISCbits.TRISC1 = 1;				/* AN5 */
//	ANSELCbits.ANSC2 = 1;				/* AN6 */
//	TRISCbits.TRISC2 = 1;				/* AN6 */
//	ANSELCbits.ANSC3 = 1;				/* AN7 */
//	TRISCbits.TRISC3 = 1;				/* AN7 */

//	A/D module
	ADCON0bits.ADON   = 1;				// On
	ADCON1bits.ADPREF = 0;				// Vref+ = Vdd
//	ADCON1bits.ADPREF = 2;				// Vref+ = External
//	ADCON1bits.ADPREF = 3;				// Vref+ = FVR
//	ADCON1bits.ADFM   = 0;				// Left jusstified
	ADCON1bits.ADFM   = 1;				// Right jusstified
//	ADCON1bits.ADCS   = 0;				// Fosc/2 Fosc=2MHz -> TAD=1us
//	ADCON1bits.ADCS   = 4;				// Fosc/4 Fosc=4MHz -> TAD=1us
	ADCON1bits.ADCS   = 1;				// Fosc/8 Fosc=8MHz -> TAD=1us
//	ADCON1bits.ADCS   = 5;				// Fosc/16 Fosc=16MHz -> TAD=1us
//	ADCON1bits.ADCS   = 2;				// Fosc/32 Fosc=32MHz -> TAD=1us
//	ADCON1bits.ADCS   = 6;				// Fosc/64 Fosc=32MHz -> TAD=2us
//	ADCON1bits.ADCS   = 3;				// Frc TAD=1-6us

// FVR module
//	FVRCONbits.FVREN = 1;				// On
//	FVRCONbits.ADFVR = 1;				// 1.024V
//	FVRCONbits.ADFVR = 2;				// 2.048V
//	FVRCONbits.ADFVR = 3;				// 4.096V
//	while(!FVRCONbits.FVRRDY);			// PIC16F193x,182x is never ready
	return;
}

void adc_stop() {
//	A/D module
	ADCON0 = 0;							// Disable
	ADCON1 = 0;
// FVR module
	FVRCON = 0;							// Disable
	return;
}

short adc_read(unsigned char channel) {
	ADCON0bits.CHS = channel;
//	__delay_us(9);						// Acquisition Time
	__delay_us(4);						// Acquisition Time
	ADCON0bits.ADGO = 1;
	while(ADCON0bits.ADGO);				// Conversion Time about 12us
	return(ADRES);
}

short adc_reads(unsigned char channel) {
	short	a;
	unsigned char	i;

	// Get a battery voltage and average
	a = 0;
	for(i = 0; i < 8; i++) {
		a += adc_read(channel);
	}
	a >>= 3;
	return(a);
}

//---------------------------------------------------------------------
//	Soft blink
//---------------------------------------------------------------------
void led_soft_blink(unsigned char cr, unsigned char cb, unsigned char cg){
	unsigned char h;

	// n wave
	for (h = 0; h < LIGHT_WAVE; h++) {
		unsigned char i;
	
		// One wave
		for (i = 0; i < (LIGHT_GRADUATION*2); i++) {
			unsigned char j;
			int pr, pg, pb;
			unsigned char a;
	
			a  = (i < LIGHT_GRADUATION) ? (i) : ((LIGHT_GRADUATION*2) - i);
			pr = dutytbl[(int)cr * a / LIGHT_GRADUATION];
			pg = dutytbl[(int)cg * a / LIGHT_GRADUATION];
			pb = dutytbl[(int)cb * a / LIGHT_GRADUATION];
	
			for (j = 0; j < LIGHT_PWM_TIME; j++) {
				int k;
				int r, g, b;
	
				r = 0;
				g = 0;
				b = 0;
				for (k = 0; k < LIGHT_PWM_PERIOD; k++) {
					if (--r < 0) {
						LED_RED_ON();
						r = pr;
					} else {
						LED_RED_OFF();
					}
	
					if (--g < 0) {
						LED_GREEN_ON();
						g = pg;
					} else {
						LED_GREEN_OFF();
					}
	
					if (--b < 0) {
						LED_BLUE_ON();
						b = pb;
					} else {
						LED_BLUE_OFF();
					}
				}
			}
		}
	}
	return;
}


//*********************************************************************
//
//	M a i n
//
//*********************************************************************
void main(){
	int 	period;
	int		duty;
	short	ontime;
	short	night_th;
	short	charge_lim;
	unsigned char	cr, cb, cg, cf;
	unsigned char	pon;

//
//		*** System clock ***
//
#if _XTAL_FREQ == 31000					//
	OSCCON =    0b00000010;				// Internal 31KHz LF
#elif _XTAL_FREQ == 31250				//
	OSCCON =    0b00010010;				// Internal 31.25KHz MF
#elif _XTAL_FREQ == 31251				//
	OSCCON =    0b00011010;				// Internal 31.25KHz HF
#elif _XTAL_FREQ == 62500				//
	OSCCON =    0b00100010;				// Internal 62.5KHz MF
#elif _XTAL_FREQ == 125000				//
	OSCCON =    0b00101010;				// Internal 125KHz MF
#elif _XTAL_FREQ == 250000				//
	OSCCON =    0b00110010;				// Internal 250KHz MF
#elif _XTAL_FREQ == 500000				//
	OSCCON =    0b00111010;				// Internal 500KHz MF
#elif _XTAL_FREQ == 125001				//
	OSCCON =    0b01000010;				// Internal 125KHz HF
#elif _XTAL_FREQ == 250001				//
	OSCCON =    0b01001010;				// Internal 250KHz HF
#elif _XTAL_FREQ == 500001				//
	OSCCON =    0b01010010;				// Internal 500KHz HF
#elif _XTAL_FREQ == 1000000				//
	OSCCON =    0b01011010;				// Internal 1MHz HF
#elif _XTAL_FREQ == 2000000				//
	OSCCON =    0b01100010;				// Internal 2MHz HF
#elif _XTAL_FREQ == 4000000				//
	OSCCON =    0b01101010;				// Internal 4MHz HF
#elif _XTAL_FREQ == 8000000				//
	OSCCON =    0b01110010;				// Internal 8MHz HF
#elif _XTAL_FREQ == 16000000			//
	OSCCON =    0b01111010;				// Internal 16MHz HF
#else									//
#error "_XTAL_FREQ value"				//
#endif									//


	//
	// Initialization
	//

	// I/O Config
	SELF_POWER_INIT();
	LED_RED_INIT();						//
	LED_GREEN_INIT();					//
	LED_BLUE_INIT();					//
#ifdef LED_GENERATOR_LOAD
	WPUA = 0;							// Pull-up all disable
	WPUC = 0;							// Pull-up all disable
	WPUCbits.WPUC2 = 1;					// LED-red C2 enable
	OPTION_REGbits.nWPUEN = 0;			// Pull-up function enable
#endif

	// Switch Mode Pump config
	period = 8;							// 1MHz PWMclock=8MHz(Fosc=8M / T2CKPS=1)
	duty = 6;							// 75% fixed duty

	//	DAC->SMP
	DACCON0bits.DACEN = 1;
	DACCON0bits.DACLPS = 0;
	DACCON0bits.DACOE = 1;
	DACCON0bits.DACPSS = 0b00;
	DACCON1bits.DACR = 9;				// Vdd/32*9 ~= 1V

	// FVR->SMP
	FVRCONbits.FVREN = 1;				// Enable
	FVRCONbits.CDAFVR =1;				// 1.024V

	// Comparator1->SMP
	CM1CON0bits.C1ON = 1;				// Enable
//	CM1CON0bits.C1OUT = 1;				// Enable
//	CM1CON0bits.C1OE = 1;				// Pin out to C1OUT/RA2
//	TRISAbits.TRISA2 = 0;				// Output port
//	CM1CON0bits.C1POL = 0;				// Polarity normal
	CM1CON0bits.C1POL = 1;				// Polarity reverse
	CM1CON0bits.C1SP = 0;				// Low-power Low-speed
//	CM1CON0bits.C1HYS = 0;				// Hysteresis disabled
	CM1CON0bits.C1HYS = 1;				// Hysteresis enabled
	CM1CON0bits.C1SYNC = 0;				// No sync timer1 clock
	CM1CON1bits.C1NCH = 1;				// Positive input = C12IN1-/RC1
	CM1CON1bits.C1PCH = 2;				// Negative input = FVR
	CM1CON1bits.C1INTP = 0;				// Interrupt on positive edge, disable
	CM1CON1bits.C1INTN = 0;				// Interrupt on negative edge, disable

	// PWM(TMR2+CCP)->SMP
	TRISCbits.TRISC5 = 1;				// Input RC5/CCP1

	PR2 = (period - 1) >> 2;			// High 8bits, Low 2bits=00

	CCP1CONbits.CCP1M = 0b1100;			// PWM mode, Active-High-High
//	CCP1CONbits.CCP1M = 0b1101;			// PWM mode, Active-High-Low
//	CCP1CONbits.CCP1M = 0b1110;			// PWM mode, Active-Low-High
//	CCP1CONbits.CCP1M = 0b1111;			// PWM mode, Active-Low-Low
	CCP1CONbits.P1M = 0b00;				// Single output
//	CCP1CONbits.P1M = 0b01;				// Full bridge forward output
//	CCP1CONbits.P1M = 0b10;				// Half bridge output
//	CCP1CONbits.P1M = 0b11;				// Full bridge reverse output

	CCP1CONbits.DC1B   = duty & 0b11;	// Low 2bits
	CCPR1L = duty >> 2;					// High 8bits

	T2CONbits.T2CKPS0 = 0b00;			// Prescaler 1/1
	T2CONbits.T2OUTPS0 = 0b0000;		// Postscaler 1/1
	T2CONbits.TMR2ON = 1;				// Start

	TRISCbits.TRISC5 = 0;				// Start PWM Output RC5/CCP1

	// PWM + Comparator -> SMP
	CCP1ASbits.CCP1AS = 0b001;			// Auto-shutdown trigger = C1
	PWM1CONbits.P1RSEN = 1;				// Auto-shutdown Restart Enable

	// ADC
	adc_start();


	//
	//	Body
	//
	ontime = LIGHT_ONTIME;
	charge_lim = CHARGE_LIM;
	cf = 0;								// Color reset
	pon = 0;
	for(;;) {
		short	ad;
		short	ab;
		unsigned char	i;

		// Wait
		__delay_ms(1000);				// Wait

		// Get an Ambient light
		LED_RED_TERM();				// Multiplex RED and AN6
		ab = adc_reads(6);			// LED = Photo senser
		LED_RED_INIT();				// Resume

		// Get a battery voltage
		ad = adc_reads(4);

		// Lower power
		if (ad < BATT_LOWLIM) {

			// Goodby sign
			LED_GREEN_OFF();
			LED_BLUE_OFF();
			for(i = 0; i < 100; i++) {
				LED_RED_REV();
				__delay_ms(200);
			}

			// Shutdown
			SELF_POWER_OFF();
			for(;;);					// Safe code

		// Over power
		} else if (ad > BATT_HIGHLIM) {

			// Start the muxmum power load
			LED_RED_ON();
			LED_GREEN_ON();
			LED_BLUE_ON();

		// Normal voltage
		} else {

			// Sunrise
			if (!pon) {					// power on 1st time
				short	l;
		
				pon = 1;
				l = (ADC_RESOLUTION - ab) * NIGHT_TH;	// Heavy code
				night_th = ab + l;
		
				eeprom_write(0x10, ab & 0xff);			// Ambient light level
				eeprom_write(0x11, ab >> 8);
				eeprom_write(0x12, night_th & 0xff);	// Threshold level
				eeprom_write(0x13, night_th >> 8);

				eeprom_write(0x00, ad & 0xff);			// Initial battery level
				eeprom_write(0x01, ad >> 8);
			}
//			eeprom_write(0x14, ab & 0xff);				// DEBUG
//			eeprom_write(0x15, ab >> 8);				// DEBUG

			// Day time
			if (ab <= night_th) {
 
				// Charge sign from green flash
				LED_RED_OFF();
				LED_GREEN_ON();
				LED_BLUE_OFF();
				__delay_ms(10);
				LED_RED_OFF();
				LED_GREEN_OFF();
				LED_BLUE_OFF();

				// Fail safe code
				if (--charge_lim <= 0) {
					RESET();
				}

			// Sunset to night
			} else {

				// Set the color of LED
				if (!cf) {
					cf = 1;
					for(i = 0; i < (COLOR_VBAT_NUM - 1); i++) {
						if (ad > vbat2code[i]) break;
					}
					cr = vbat2red[i];
					cg = vbat2green[i];
					cb = vbat2blue[i];

					eeprom_write(0x02, ad & 0xff);	// Light on battery level
					eeprom_write(0x03, ad >> 8);
				}

				// Light
				led_soft_blink(cr, cg, cb);

				// Timeout
				if (--ontime <= 0) {
					eeprom_write(0x04, ad & 0xff);	// Light off battery level
					eeprom_write(0x05, ad >> 8);

					SELF_POWER_OFF();	// Shutdown
					for(;;);			// Safe code
				}
			}
		}

	}
//	adc_stop();
//	return;
}